"""
Models for the splash screen application
"""

from django.db import models

from config_models.models import ConfigurationModel


class SplashConfig(ConfigurationModel):
    """
    Configuration for the splash django app
    """
    cookie_name = models.TextField(
        default=u'edx_splash_screen',
        help_text=u"The name of the cookie to check when assessing if the user needs to be redirected"
    )
    cookie_allowed_values = models.TextField(
        default=u'seen',
        help_text=u"Comma-separated list of values accepted as cookie values to prevent the redirect"
    )
    unaffected_usernames = models.TextField(
        default=u'',
        blank=True,
        help_text=u"Comma-separated list of users which should never be redirected (usernames)"
    )
    unaffected_url_paths = models.TextField(
        default=u'',
        blank=True,
        help_text=u"Comma-separated list of URL paths (not including the hostname) which should not be redirected. "
                  u"Paths may include wildcards denoted by * (example: /*/student_view)"
    )
    redirect_url = models.URLField(
        default=u'http://edx.org',
        help_text=u"The URL the users should be redirected to when they don't have the right cookie"
    )

    @property
    def cookie_allowed_values_list(self):
        """
        `cookie_allowed_values` as a list of string values
        """
        if not self.cookie_allowed_values.strip():
            return []

        return [val.strip() for val in self.cookie_allowed_values.split(',')]

    @property
    def unaffected_usernames_list(self):
        """
        `unaffected_usernames` as a list of username values
        """
        if not self.unaffected_usernames.strip():
            return []

        return [name.strip() for name in self.unaffected_usernames.split(',')]

    @property
    def unaffected_url_paths_list(self):
        """
        `unaffected_url_paths` as a list of URL paths values
        """
        if not self.unaffected_url_paths.strip():
            return []

        return [url.strip() for url in self.unaffected_url_paths.split(',')]

    def save(self, *args, **kwargs):  # pylint: disable=arguments-differ
        """Call `full_clean` before saving to ensure proper validation of configuration values"""
        self.full_clean()
        super(SplashConfig, self).save(*args, **kwargs)



"""
Splash screen - Middleware
"""
import logging
import re

from django.shortcuts import redirect
from django.utils.deprecation import MiddlewareMixin

from .models import SplashConfig

log = logging.getLogger(__name__)


class SplashMiddleware(MiddlewareMixin):
    """
    Checks incoming requests, to redirect users to a configured splash screen URL
    if they don't have the proper cookie set

    This can be used to display a small marketing landing page, protect an
    alpha website from the public eye, make an announcement, etc.
    """

    def process_request(self, request):
        """
        Determine if the user needs to be redirected
        """
        config = SplashConfig.current()
        if not config.enabled:
            return None

        # Some URLs should never be redirected
        path_info = request.path_info
        for unaffected_path in config.unaffected_url_paths_list:
            if self.path_matches(path_info, unaffected_path):
                return None

        # Some users should never be redirected
        if request.user.username in config.unaffected_usernames_list:
            return None

        cookie_value = request.COOKIES.get(config.cookie_name)
        if (cookie_value not in config.cookie_allowed_values_list and
                request.build_absolute_uri() != config.redirect_url):
            return redirect(config.redirect_url)
        return None

    def path_matches(self, path, pattern):
        """
        Determine whether `path` matches the `pattern`.

        `pattern` may include wildcards (*) which represent a sequence of
        zero or more arbitrary characters.
        """
        matches = False

        if path == pattern:
            matches = True
        elif '*' in pattern:
            pattern = re.escape(pattern).replace('\\*', '.*')
            if re.match(pattern + '$', path):
                matches = True

        return matches



"""
Admin site bindings for splash screen
"""


from django.contrib import admin

from config_models.admin import ConfigurationModelAdmin

from splash.models import SplashConfig

admin.site.register(SplashConfig, ConfigurationModelAdmin)
